@@ -17,9 +17,7 @@
    Red Hat Authors:
         hdegoede@redhat.com
 */
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <string.h>
 #include <spice/vd_agent.h>
