@@ -35,21 +35,25 @@
 
 #include "spice.h"
 #include "utils.h"
+#include "sys-socket.h"
 
 #define SPICE_UPCAST(type, ptr) \
     (verify_expr(SPICE_OFFSETOF(type, base) == 0,SPICE_CONTAINEROF(ptr, type, base)))
 
+SPICE_BEGIN_DECLS
+
+void red_timer_start(SpiceTimer *timer, uint32_t ms);
+void red_timer_cancel(SpiceTimer *timer);
+void red_timer_remove(SpiceTimer *timer);
+void red_watch_update_mask(SpiceWatch *watch, int event_mask);
+void red_watch_remove(SpiceWatch *watch);
+
 typedef struct SpiceCoreInterfaceInternal SpiceCoreInterfaceInternal;
 
 struct SpiceCoreInterfaceInternal {
     SpiceTimer *(*timer_add)(const SpiceCoreInterfaceInternal *iface, SpiceTimerFunc func, void *opaque);
-    void (*timer_start)(const SpiceCoreInterfaceInternal *iface, SpiceTimer *timer, uint32_t ms);
-    void (*timer_cancel)(const SpiceCoreInterfaceInternal *iface, SpiceTimer *timer);
-    void (*timer_remove)(const SpiceCoreInterfaceInternal *iface, SpiceTimer *timer);
 
     SpiceWatch *(*watch_add)(const SpiceCoreInterfaceInternal *iface, int fd, int event_mask, SpiceWatchFunc func, void *opaque);
-    void (*watch_update_mask)(const SpiceCoreInterfaceInternal *iface, SpiceWatch *watch, int event_mask);
-    void (*watch_remove)(const SpiceCoreInterfaceInternal *iface, SpiceWatch *watch);
 
     void (*channel_event)(const SpiceCoreInterfaceInternal *iface, int event, SpiceChannelEventInfo *info);
 
@@ -70,6 +74,7 @@ struct SpiceCoreInterfaceInternal {
 };
 
 extern const SpiceCoreInterfaceInternal event_loop_core;
+extern const SpiceCoreInterfaceInternal core_interface_adapter;
 
 typedef struct RedsState RedsState;
 
@@ -78,6 +83,10 @@ typedef struct GListIter {
     GList *next;
 } GListIter;
 
+/* Iterate through a GList. Note that the iteration is "safe" meaning that the
+ * current item can be removed while the list is scanned. This is required as
+ * the code inside the loop in some cases can remove the element we are
+ * processing */
 #define GLIST_FOREACH_GENERIC(_list, _iter, _type, _data, _dir) \
     for (GListIter _iter = { .link = _list }; \
         (_data = (_type *) (_iter.link ? _iter.link->data : NULL), \
@@ -120,4 +129,6 @@ typedef struct GListIter {
     { return G_TYPE_INSTANCE_GET_CLASS(obj, \
              module_obj_name ## _get_type(), ModuleObjName ## Class); }
 
+SPICE_END_DECLS
+
 #endif /* RED_COMMON_H_ */
